/************** BEST VIEWED IN A CODE EDITOR 80 COLUMNS WIDE *******************
*
* Dewalt ToughCase Pegs
* Benjamen Johnson <workshop.electronsmith.com>
* 20211201
* openSCAD Version: 2019.05 
*******************************************************************************/
/*[Hidden]*/
ver = "ELECTRONSMITH V1.1";
// imperial drive sizes converted to mm
drive_sizes = [6.35,9.53,12.7];

// Depths of the difference sizes of case
total_depth = [35,28,36.5];

// add a little to make cuts render correctly
$dl=0.01;

// Control how smooth curves are
$fa=11.25;
$fs=0.1;   

/*[Choose]*/
/*******************************************************************************
*******************************************************************************/
// What are you making?
type = "Square Drive Peg"; // [Square Drive Peg, Square Drive Pin, OMT Peg, Grinder Peg]

case = 0; // [0:Small, 1:Medium, 2:Large]
/*[Peg Hole Parameters]*/
/*******************************************************************************
*******************************************************************************/
// Inner Diameter of the peg hole
inner_dia = 5.25;

// Outer Diameter of the peg Hole
outer_dia = 8.2;

// How many degrees in each lobe
lobe_angle = 95;

// Additional degrees to add to make peg lock
locking_angle = 45;

// How much to cut out of base to make a peg lock
cutout_depth=3.0;

// Thickness of the baseplate
base_depth = 4.9;

/*[Peg Parameters]*/
/*******************************************************************************
*******************************************************************************/
//How much of the peg to cut out
cutout_angle = 30;

// angular size of the peg -- smaller than the hole
peg_lobe_angle = 85;

//Size of the gap in the locking peg
gap = 2.2;

// Adjust size of the peg
peg_fit = 0.95; //[0.95:0.05:1.05]

// Adjustment to make pegs line up square 
peg_locking_angle = -5; //[-45:1:45]

// Create supports -- much better than slicer supports
use_supports=true;

/*[Square Drive Parameters]*/
/******************************************************************************
* Square Drive Parameters
*******************************************************************************/
// Choose the size of the socket drive
drive_type =1; //[0:1/4 Drive, 1:3/8 Drive, 2:1/2 Drive]

// Adjust the fit of the stub
drive_fit = 0.99; //[0.95:0.01:1.05]

// calcualte actual drie size
drive_size = drive_sizes[drive_type]*drive_fit;

// Size of hole in the center for pin
drive_hole_dia = 6;

// How much material to leave in the bottom of the drive hole,
wall_thickness = 3;

// Height of the stub that holds the socket
stub_height = 12;

// How much to chamfer the top of the stub
drive_chamfer = 1;

// How much to chamfer the sides
side_chamfer = 3;

/*[Pin Parameters]*/
/******************************************************************************
* Pin Parameters
*******************************************************************************/
// Adjust how tight the pin fits
pin_fit = 0.98; //[0.95:0.01:1.05]

// Create a label or not
label = true;

// position of the label 
x_pos = 9; //[-10:0.5:40]

// Label Text
label_text = "1/2";

// Label Font
label_font = "Ariel";

// Label Font Size
font_size = 5;

// Label Font Thickness
font_thickness = 0.4;

/*[OMT Parameters]*/
/******************************************************************************
* The posts are what hold the OMT blades
*******************************************************************************/
// Configuration of the wings that hold the blades from rotating
key = "4 Keys"; //[No Keys, 2 Keys, 3 Keys, 4 Keys, 6 Keys]

key_orientation = 0;

// Diameter of the post
OMT_post_dia = 9;

OMT_standoff_height = 8;

// Post height
OMT_post_height = total_depth[case]-base_depth;

// Thickness of the keys that stop the blade from rotating
key_thickness = 1.5;

// Width of the keys that stop the blade from rotating
key_width = 3;

function even(a) = a%2;

/*******************************************************************************
* Choose which part to make
*******************************************************************************/
if(type == "Square Drive Peg")
    translate([0,0,stub_height+base_depth])
    rotate([180,0,0])
    make_square_drive();
else if(type == "Square Drive Pin")
    translate([0,0,17.5])
    rotate([180,0,0])
    make_square_drive_pin();
else if (type == "OMT Peg")
    translate([0,0,total_depth[case]])
    rotate([180,0,0])
    make_OMT_peg();
else if (type == "Grinder Peg")
    translate([0,0,total_depth[case]])
    rotate([180,0,0])
    make_grinder_peg();

/*******************************************************************************
* Make the part the that OMT blades fit onto
*******************************************************************************/
module make_OMT_peg(){

inside_depth = total_depth[case];
transition_dia = OMT_post_dia+2*key_width;

    union(){
        // locking base
        make_locking_base();
        
        // OMT Post
        translate([0,0,OMT_post_height/2+base_depth])
        rotate([90,0,0])
        OMT_connector();
        
        // Transition to a wider base
        translate([0,0,OMT_standoff_height+base_depth])
        cylinder(d1=transition_dia,d2=OMT_post_dia,h=key_width);
        
        translate([0,0,base_depth])
        cylinder(d=transition_dia,h=OMT_standoff_height);
    } //end union
}// end module

/*******************************************************************************
* Make square drive peg
*******************************************************************************/
module make_square_drive(){

    total_height = stub_height+base_depth;
    top_scale=(drive_size-2*drive_chamfer)/drive_size;
    
    //make base
    make_locking_base();
    
    union(){
        //make stub    
        difference(){
            translate([0,0,base_depth])
                union(){
                    translate([0,0,stub_height-drive_chamfer])
                    linear_extrude(height=drive_chamfer,$fn=200,scale=top_scale)
                        drive_shape();
                    linear_extrude(height=stub_height-drive_chamfer,$fn=200)
                        drive_shape();
                }//end union
            translate([0,0,base_depth+wall_thickness])
            cylinder(d=drive_hole_dia,h=stub_height,$fn=25);
        }//end difference
        
        //make skirt
        skirt_height = 1.3;
        translate([0,0,skirt_height/2+base_depth])
        cylinder(d1=drive_size+side_chamfer, d2=drive_size, h=skirt_height,center=true, $fn=200);
    } //end union
    
} //end module  

/*******************************************************************************
* Make shape of the socket drive
*******************************************************************************/
module make_square_drive_pin(){
    
    pin_dia = drive_hole_dia*pin_fit;
    pin_height = total_depth[case]-wall_thickness-base_depth; 

    translate([-15,0,15])
    union(){
        rotate([90,0,0])
        difference(){
            // make pin shape
            cylinder(d=pin_dia,h=pin_height,$fn=25,center=true);
            
            //shave off back for a flat place to print
            translate([0,pin_dia-0.5,0])
            cube([pin_dia,pin_dia,pin_height+$dl],center=true);
            
            // make flat surface for text
            translate([0,-pin_dia/2-1,-stub_height+
            wall_thickness])
            cube([pin_dia,pin_dia,pin_height],center=true);
        } // end difference
        
        if (label)
            translate([0,x_pos,-1])
            color("white")
            rotate([0,180,90])
            linear_extrude(height = font_thickness)
                text(text = label_text, font = label_font, size = font_size, valign="center",halign = "center");

    }//end union
} //end module

/*******************************************************************************
* Make shape of the socket drive
*******************************************************************************/
module drive_shape(){
    intersection(){
        square(drive_size,center=true);
        circle(d=drive_size+side_chamfer);
    } //end intersection
        
} //end module

/*******************************************************************************
* Make hole saw peg
*******************************************************************************/
module make_grinder_peg(){
    
inside_depth = total_depth[packout_height];
    
    // make locking shape
    make_locking_base();
    
    translate([0,0,base_depth+(inside_depth-base_depth)/2]) 
    cylinder(d=21,h=inside_depth-base_depth,center=true,$fn=50);

} //end module

/*******************************************************************************
* Make the locking base of the pegs
*******************************************************************************/
module make_locking_base(){

support_dia = 1;
peg_inner_dia = inner_dia*peg_fit;
peg_outer_dia = outer_dia*peg_fit;
support_x_pos = (peg_outer_dia-support_dia)/2;

    rotate([0,0,peg_locking_angle])
    union(){
        shape(base_depth-gap,peg_lobe_angle,peg_inner_dia,peg_outer_dia);
        translate([0,0,base_depth-gap])
        rotate([0,0,27.5])
        shape(gap,cutout_angle,peg_inner_dia,peg_outer_dia);
        
        if(use_supports){
            // create supports
            rotate([0,0,-34])
            translate([support_x_pos,0,base_depth-gap/2])
            cylinder(d=support_dia,h=gap,center=true);
            
            rotate([0,0,-34])
            translate([-support_x_pos,0,base_depth-gap/2])
            cylinder(d=support_dia,h=gap,center=true);
        } //end if
    } //end union
}// end module

/*******************************************************************************
* Module for making the strange shape in 2D to speed rendering
*******************************************************************************/
module flat_shape(angle,i,o){
    union(){
        arc(o, angle);
        
        rotate(180)
        arc(o, angle);
        
        circle(d=i);
    }//end union
} //end module

/*******************************************************************************
* Module for making the strange shape
*******************************************************************************/
module shape(h,angle,i,o){
    linear_extrude(height=h,convexity=10)
        union(){
            arc(o, angle);
            
            rotate(180)
            arc(o, angle);
            
            circle(d=i);
        }//end union
} //end module

/*******************************************************************************
* Module for creating an arc
*******************************************************************************/
module arc(o,a){
    rotate(180)
    difference(){
        circle(d=o);
        wedge(outer_dia,360-a);
    } //end_difference
} //end module arc

/*******************************************************************************
* Module for creating the wedge shape to subtract from a circle to make an arc
*******************************************************************************/
module wedge(x=1,a=45){

t = a/2;
triangle=[[0,0],[x*cos(t),x*sin(t)],[x*cos(t),-x*sin(t)]];
    
    if(a<=90)
        polygon(triangle);
    else{
        rotate(t-90)square(x);
        rotate(-t)square(x);
        if(a>180) //fill in gap
            rotate(-45)square(x);
        if(a>270){ //fill in gap
            
            rotate(45)square(x);
            rotate(-135)square(x);
        } //end if
    } //end else
} //end module wedge

/*******************************************************************************
* Make the part the that OMT blades fit ontp
*******************************************************************************/
module OMT_connector(){
    
    union(){
        rotate([90,0,0])
        cylinder(d=OMT_post_dia,h=OMT_post_height,center=true);
        
        rotate([0,key_orientation,0])
        union(){    
            if((key=="2 Keys") || (key=="3 Keys") || (key =="4 Keys"))
            cube([OMT_post_dia+2*key_width,OMT_post_height,key_thickness],center=true);
            
            if((key=="3 Keys") || (key =="4 Keys"))    
            translate([0,0,-(OMT_post_dia/2+key_width/2)+1])
            cube([key_thickness,OMT_post_height,key_width+2],center=true);
            
            if(key =="4 Keys") 
            translate([0,0,(OMT_post_dia/2+key_width/2)-1])
            cube([key_thickness,OMT_post_height,key_width+2],center=true);
          
            if(key =="6 Keys"){
                cube([OMT_post_dia+2*key_width,OMT_post_height,key_thickness],center=true);
                rotate([0,120,0])
                cube([OMT_post_dia+2*key_width,OMT_post_height,key_thickness],center=true);
                rotate([0,240,0])
                cube([OMT_post_dia+2*key_width,OMT_post_height,key_thickness],center=true);
            }//end if
            
        }//end union  
    } //end union
} // end module
